from django.utils import translation

from migration_fixer.utils import _clean_message, _decode_message, no_translations


def test__clean_message():
    message = "New\n\rTest\n\r"
    expected_output = "New\n\rTest"

    current_output = _clean_message(message)

    assert expected_output == current_output


def test__decode_message():
    encoding = "utf-8"
    message = b"\xe2\x86\x92"
    expected_output = "→"

    current_output = _decode_message(message, encoding)

    assert expected_output == current_output


def test_no_translations():
    @no_translations
    def do_not_translate():
        assert translation.get_language() is None

    do_not_translate()



DEFAULT_BRANCH = "main"
TEST_01_MIGRATION_BRANCH = "feature/migration-test-01"  # Single merge to main
TEST_02_MIGRATION_BRANCH = (
    "feature/migration-test-02"  # Single merge to main after 01 is merged
)
TEST_03_MIGRATION_BRANCH = (
    "feature/migration-test-03"  # Merge 04 first before merging this to main
)
TEST_04_MIGRATION_BRANCH = (
    "feature/migration-test-04"  # Merge into 04 default should be 03
)



import os
import pathlib

import pytest
from git import Repo
from pytest_shutil.workspace import Workspace


@pytest.fixture(autouse=True)
def use_demo_app(settings):
    settings.INSTALLED_APPS = [
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "migration_fixer",
        "migration_fixer.tests.demo",
    ]


class GitRepo(Workspace):
    def __init__(self, workspace, delete=False):
        super(GitRepo, self).__init__(workspace, delete)
        self.api = Repo.init(self.workspace)
        self.uri = "file://%s" % self.workspace


@pytest.fixture
def git_repo(settings):
    workspace = pathlib.Path(
        os.path.join(settings.BASE_DIR, "migration_fixer", "tests", "demo")
    )

    with GitRepo(workspace=workspace) as repo:
        yield repo



"""Tests for `migration_fixer` package."""

import pytest

from migration_fixer.management.commands.makemigrations import Command
from migration_fixer.tests.management.commands.constants import (
    TEST_01_MIGRATION_BRANCH,
    TEST_02_MIGRATION_BRANCH,
)
from migration_fixer.tests.management.commands.utils import (
    execute_command,
    temporary_checkout,
)


@pytest.mark.env("test_01")
@pytest.mark.django_db
def test_run_makemigrations_is_valid_without_any_conflicts(git_repo):
    cmd = Command(repo=git_repo.api)

    with temporary_checkout(
        git_repo,
        default_branch_name=TEST_01_MIGRATION_BRANCH,
        target_branch_name=TEST_01_MIGRATION_BRANCH,
    ) as target_branch:
        output1 = execute_command(cmd)
        output2 = execute_command(cmd, fix=True)

    assert target_branch.name == TEST_01_MIGRATION_BRANCH
    assert output1 == "No changes detected\n"
    assert output2 == "No changes detected\n"


@pytest.mark.env("test_01")
@pytest.mark.django_db
def test_run_makemigrations_is_valid_without_any_conflicts_verbose(git_repo):
    cmd = Command(repo=git_repo.api)

    with temporary_checkout(
        git_repo,
        default_branch_name=TEST_01_MIGRATION_BRANCH,
        target_branch_name=TEST_01_MIGRATION_BRANCH,
    ) as target_branch:
        output1 = execute_command(cmd, verbosity=2)
        output2 = execute_command(cmd, verbosity=2, fix=True)

    assert target_branch.name == TEST_01_MIGRATION_BRANCH
    assert output1 == "No changes detected\n"
    assert output2 == "No changes detected\n"


@pytest.mark.env("test_02")
@pytest.mark.django_db
def test_run_makemigrations_with_fix_is_valid_for_conflicts(git_repo):
    cmd = Command(repo=git_repo.api)

    with temporary_checkout(
        git_repo,
        default_branch_name=TEST_01_MIGRATION_BRANCH,
        target_branch_name=TEST_02_MIGRATION_BRANCH,
    ) as target_branch:
        output = execute_command(cmd, default_branch=TEST_01_MIGRATION_BRANCH, fix=True)

    assert target_branch.name == TEST_02_MIGRATION_BRANCH
    assert output == f"{cmd.success_msg}\n"


@pytest.mark.env("test_02")
@pytest.mark.django_db
def test_run_makemigrations_with_fix_is_valid_for_conflicts_verbose(git_repo):
    cmd = Command(repo=git_repo.api)
    expected_output = f"""Verifying git repository...
Retrieving the current branch...
Fetching git remote origin changes on: {TEST_01_MIGRATION_BRANCH}
Retrieving the last commit sha on: {TEST_01_MIGRATION_BRANCH}
Retrieving changed files between the current branch and {TEST_01_MIGRATION_BRANCH}
Retrieving the last migration on: {TEST_01_MIGRATION_BRANCH}
Fixing numbered migration...
Successfully fixed migrations.
"""

    with temporary_checkout(
        git_repo,
        default_branch_name=TEST_01_MIGRATION_BRANCH,
        target_branch_name=TEST_02_MIGRATION_BRANCH,
    ) as target_branch:
        output = execute_command(
            cmd, default_branch=TEST_01_MIGRATION_BRANCH, verbosity=2, fix=True
        )

    assert target_branch.name == TEST_02_MIGRATION_BRANCH
    assert output == expected_output
